﻿#include <iostream>
#include <vector>
#include <cmath>
#include <cstdlib>
#include <ctime>
#include <limits>

using namespace std;

// 定义点的结构体
struct Point {
    double x;
    double y;
};

// 计算两个点之间的欧几里德距离
static double calculateDistance(const Point& p1, const Point& p2) {
    return sqrt(pow(p1.x - p2.x, 2) + pow(p1.y - p2.y, 2));
}

vector<Point> initializeCentroids(int k, const vector<Point>& data) {
    vector<Point> centroids;
    srand(time(0));

    for (int i = 0; i < k; ++i) {//随机选择k个点为初始点
        int index = rand() % data.size();
        centroids.push_back(data[index]);
    }

    return centroids;
}

// 执行K均值算法
vector<vector<Point>> kMeans(int k, const vector<Point>& data) {
    // 初始化簇的中心
    vector<Point> centroids = initializeCentroids(k, data);

    // 存储每个簇的数据点
    vector<vector<Point>> clusters(k);

    // 迭代次数，这里简单设置为10次，你可以根据需要调整
    for (int iteration = 0; iteration < 10; ++iteration) {
        // 将每个数据点分配到最近的簇
        for (const Point& point : data) {
            double minDistance = numeric_limits<double>::max();
            int closestCentroid = 0;

            for (int i = 0; i < k; ++i) {
                double distance = calculateDistance(point, centroids[i]);
                if (distance < minDistance) {
                    minDistance = distance;
                    closestCentroid = i;
                }
            }

            clusters[closestCentroid].push_back(point);
        }

        // 更新簇的中心为数据点的平均值
        for (int i = 0; i < k; ++i) {
            if (!clusters[i].empty()) {
                double sumX = 0.0;
                double sumY = 0.0;

                for (const Point& point : clusters[i]) {
                    sumX += point.x;
                    sumY += point.y;
                }

                centroids[i].x = sumX / clusters[i].size();
                centroids[i].y = sumY / clusters[i].size();
                cout << "当前中心点为：" << centroids[i].x << " " << centroids[i].y << endl;
            }
        }

        // 清空簇中的数据点，准备下一轮迭代
        if (iteration != 9) {
            for (int i = 0; i < k; ++i) {
                clusters[i].clear();
            }
        }
    }

    return clusters;
}

int main() {
    // 输入K、事务数量n以及n个对象
    int k, n;
    cout << "输入k值: ";
    cin >> k;
    cout << "输入n值: ";
    cin >> n;

    vector<Point> data(n);

    cout << "输入" << n << "个事务：" << endl;
    for (int i = 0; i < n; ++i) {
        cout << "Point " << i + 1 << ": ";
        cin >> data[i].x >> data[i].y;
    }

    // 执行K均值算法
    vector<vector<Point>> clusters = kMeans(k, data);

    // 输出簇的结果
    cout << "\nClusters:" << endl;
    for (int i = 0; i < k; ++i) {
        cout << "Cluster " << i + 1 << ": ";
        for (const Point& point : clusters[i]) {
            cout << "(" << point.x << ", " << point.y << ") ";
        }
        cout << endl;
    }

    return 0;
}
/*

3
8
1 1
2 1
5 3
4 4 
5 4
1 2
2 2
4 3

*/